<main class="container">
    <article>
        <h1>{{error_status_code}}</h1>
        <p>{{error_message}}</p>
        <footer>
            <a role="button" class="primary" href="/">🏠 Go Back</a>
        </footer>
    </article>
</main>